# frozen_string_literal: true

module Bulkrax
  # Import Behavior for Entry classes
  module ImportBehavior # rubocop:disable Metrics/ModuleLength
    extend ActiveSupport::Concern

    def build_for_importer
      begin
        build_metadata
        unless importerexporter.validate_only
          raise CollectionsCreatedError unless collections_created?

          @item = factory.run!

          add_user_to_permission_templates! if self.class.to_s.include?('Collection')
          parent_jobs if parsed_metadata[related_parents_parsed_mapping]&.join.present?
          child_jobs if parsed_metadata[related_children_parsed_mapping]&.join.present?
        end
      rescue RSolr::Error::Http, CollectionsCreatedError => e
        raise e
      rescue StandardError => e
        status_info(e)
      else
        status_info
      ensure
        save!
      end
      @item
    end

    def add_user_to_permission_templates!
      permission_template = Hyrax::PermissionTemplate.find_or_create_by!(source_id: @item.id)
      access_grants_attributes.each do |grant|
        permission_template_access = Hyrax::PermissionTemplateAccess.find_or_initialize_by(
          permission_template_id: permission_template.id.to_s,
          agent_type: grant[:agent_type],
          agent_id: grant[:agent_id],
          access: grant[:access]
        )

        permission_template_access.is_system_generated = grant[:is_system_generated] if grant[:is_system_generated]
        permission_template_access.save
      end

      @item.reset_access_controls!
    end

    def access_grants_attributes
      agents = [
        { agent_type: 'group', agent_id: 'admin', access: Hyrax::PermissionTemplateAccess::MANAGE,
          is_system_generated: true },
        { agent_type: 'user', agent_id: user.user_key, access: Hyrax::PermissionTemplateAccess::MANAGE,
          is_system_generated: true }
      ]

      if @item.collection_type.title == 'CRC 1280'
        agents << { agent_type: 'group',
                    agent_id: "#{@item.collection_type.title.parameterize(separator: '_')}_manager", access: Hyrax::PermissionTemplateAccess::MANAGE, is_system_generated: true }
        agents << { agent_type: 'group',
                    agent_id: "#{@item.collection_type.title.parameterize(separator: '_')}_member", access: Hyrax::PermissionTemplateAccess::VIEW, is_system_generated: true }

        %w[manager member].each do |role_type|
          role = Role.find_or_initialize_by(name: "#{@item.collection_type.title.parameterize(separator: '_')}_#{role_type}")

          role.is_system_generated = true

          role.is_crc_manager = true if role_type == 'manager'
          role.is_crc_member = true if role_type == 'member'

          role.save
        end

        if @item.crc_collection_type == 'group'
          agents << { agent_type: 'group',
                      agent_id: "#{@item.collection_type.title.parameterize(separator: '_')}_#{@item.id}_manager", access: Hyrax::PermissionTemplateAccess::MANAGE, is_system_generated: true }

          %w[manager member].each do |role_type|
            if role_type == 'member'
              agents << { agent_type: 'group',
                          agent_id: "#{@item.collection_type.title.parameterize(separator: '_')}_#{@item.id}_#{role_type}", access: Hyrax::PermissionTemplateAccess::DEPOSIT, is_system_generated: true }
            end

            role = Role.find_or_initialize_by(name: "#{@item.collection_type.title.parameterize(separator: '_')}_#{@item.id}_#{role_type}")

            role.is_system_generated = true

            role.is_crc_group_manager = true if role_type == 'manager'
            role.is_crc_group_member = true if role_type == 'member'

            role.save
          end
        end
      end

      agents
    end

    def parent_jobs
      parsed_metadata[related_parents_parsed_mapping].each do |parent_identifier|
        next if parent_identifier.blank?

        PendingRelationship.create!(child_id: identifier, parent_id: parent_identifier,
                                    importer_run_id: importerexporter.last_run.id, order: id)
      end
    end

    def child_jobs
      parsed_metadata[related_children_parsed_mapping].each do |child_identifier|
        next if child_identifier.blank?

        PendingRelationship.create!(parent_id: identifier, child_id: child_identifier,
                                    importer_run_id: importerexporter.last_run.id, order: id)
      end
    end

    def find_collection_ids
      collection_ids
    end

    # override this in a sub-class of Entry to ensure any collections have been created before building the work
    def collections_created?
      true
    end

    def build_metadata
      raise StandardError, 'Not Implemented'
    end

    def rights_statement
      parser.parser_fields['rights_statement']
    end

    # try and deal with a couple possible states for this input field
    def override_rights_statement
      %w[true 1].include?(parser.parser_fields['override_rights_statement'].to_s)
    end

    def add_rights_statement
      if override_rights_statement || parsed_metadata['rights_statement'].blank?
        parsed_metadata['rights_statement'] = [parser.parser_fields['rights_statement']]
      end
    end

    def add_visibility
      parsed_metadata['visibility'] = importerexporter.visibility if parsed_metadata['visibility'].blank?
    end

    def add_admin_set_id
      parsed_metadata['admin_set_id'] = importerexporter.admin_set_id if parsed_metadata['admin_set_id'].blank?
    end

    def add_collections
      return if find_collection_ids.blank?

      parsed_metadata['member_of_collections_attributes'] = {}
      find_collection_ids.each_with_index do |c, i|
        parsed_metadata['member_of_collections_attributes'][i.to_s] = { id: c }
      end
    end

    # Attempt to sanitize Questioning Authority URI values for configured controlled fields of common
    # data entry mistakes. Controlled URI values are only valid if they are an exact match.
    # Example:
    #   Valid value:     http://rightsstatements.org/vocab/InC/1.0/
    #   Provided value:  https://rightsstatements.org/vocab/InC/1.0
    #   Sanitized value: http://rightsstatements.org/vocab/InC/1.0/ ("s" from "https" removed, trailing "/" added)
    #
    # @return [Boolean] true if all controlled URI values are sanitized successfully
    def sanitize_controlled_uri_values!
      Bulkrax.qa_controlled_properties.each do |field|
        next if parsed_metadata[field].blank?

        parsed_metadata[field].each_with_index do |value, i|
          next if value.blank?

          if (validated_uri_value = validate_value(value, field))
            parsed_metadata[field][i] = validated_uri_value
          else
            debug_msg = %(Unable to locate active authority ID "#{value}" in config/authorities/#{field.pluralize}.yml)
            Rails.logger.debug(debug_msg)
            error_msg = %("#{value}" is not a valid and/or active authority ID for the :#{field} field)
            raise ::StandardError, error_msg
          end
        end
      end

      true
    end

    # @param value [String] value to validate
    # @param field [String] name of the controlled property
    # @return [String, nil] validated URI value or nil
    def validate_value(value, field)
      if value.match?(::URI::DEFAULT_PARSER.make_regexp)
        value = value.strip.chomp
        # add trailing forward slash unless one is already present
        value << '/' unless value.match?(%r{/$})
      end

      valid = if active_id_for_authority?(value, field)
                true
              else
                value.include?('https') ? value.sub!('https', 'http') : value.sub!('http', 'https')
                active_id_for_authority?(value, field)
              end

      valid ? value : nil
    end

    # @param value [String] value to check
    # @param field [String] name of the controlled property
    # @return [Boolean] provided value is a present, active authority ID for the provided field
    def active_id_for_authority?(value, field)
      field_service = "Hyrax::#{"#{field}_service".camelcase}".constantize
      active_authority_ids = field_service.new.active_elements.map { |ae| ae['id'] }

      active_authority_ids.include?(value)
    end

    def factory
      @factory ||= Bulkrax::ObjectFactory.new(attributes: parsed_metadata,
                                              source_identifier_value: identifier,
                                              work_identifier: parser.work_identifier,
                                              related_parents_parsed_mapping: parser.related_parents_parsed_mapping,
                                              replace_files: replace_files,
                                              user: user,
                                              klass: factory_class,
                                              importer_run_id: importerexporter.last_run.id,
                                              update_files: update_files)
    end

    def factory_class
      fc = if parsed_metadata&.[]('model').present?
             parsed_metadata&.[]('model').is_a?(Array) ? parsed_metadata&.[]('model')&.first : parsed_metadata&.[]('model')
           elsif mapping&.[]('work_type').present?
             parsed_metadata&.[]('work_type').is_a?(Array) ? parsed_metadata&.[]('work_type')&.first : parsed_metadata&.[]('work_type')
           else
             Bulkrax.default_work_type
           end

      # return the name of the collection or work
      fc.tr!(' ', '_')
      fc.downcase! if fc.match?(/[-_]/)
      fc.camelcase.constantize
    rescue NameError
      nil
    rescue StandardError
      Bulkrax.default_work_type.constantize
    end
  end
end
